<?php
namespace PME\Controllers\Cardapio;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class CopiaPratoEComensais extends \PME\Services\TeknisaCrud {
	protected $connection;
	protected $environment;
	protected $utilitiesCard;
	protected $elaboracaoCardapio;

    public function __construct($crudService, $parameterBag, $connection, $environment, $utilitiesCard, $elaboracaoCardapio) {
        parent::__construct($crudService, $parameterBag);
        $this->connection = $connection;
        $this->environment = $environment;
        $this->utilitiesCard = $utilitiesCard;
        $this->elaboracaoCardapio = $elaboracaoCardapio;
	}
	
	public function realizaAcaoCopiaPratos(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $cdfilcentpme = $row['PRATOSELECIONADO']['CDFILCENTPME'];
            $cdtipocard = $row['CDTIPOCARD'];
            $cdservfili = $row['PRATOSELECIONADO']['CDSERVFILI'];
            $cdfilial = $row['PRATOSELECIONADO']['CDFILIAL'];
            $cdprat = $row['PRATOSELECIONADO']['CDPRATO'];
            $cdfilauxprat = $row['PRATOSELECIONADO']['CDFILAUXPRAT'];
            $cdtipoprat = $row['PRATOSELECIONADO']['CDTIPOPRAT'];
            $nrcardpme = $row['PRATOSELECIONADO']['NRCARDPME'];
            $nrordcnfcpme = $row['PRATOSELECIONADO']['NRORDPRACPME'];
            $nrseqcnfcpm = $row['PRATOSELECIONADO']['NRSEQPRACPME'];
            $periodoCard = $row['PERIODOCARD'];
            $excecoes = $row['EXCECOES'];
            $comensaisNovosDoPrato = $row['COMENSAISPRATO'];
            $sobreporValue = $row['SOBREPOR'];
            
            $comensaisDiaOuPrato = ($row['RADIOVALUE']==0) ? 'dia' : 'prato';
            $possuiAgregadaEsp = ($row['PRATOSELECIONADO']['filterData']['aggregated']) ? true : false;
            $filial = $possuiAgregadaEsp ? $cdfilial : NULL;
            
            $comensaisDoDia = $this->buscaComensaisDiariosDaSemana($cdfilcentpme, $cdtipocard, $cdservfili, $filial); // busco os comensais do dia
            $arrayComensais = ['QTCOMPMEDOM', 'QTCOMPMESEG', 'QTCOMPMETER', 'QTCOMPMEQUA', 'QTCOMPMEQUI', 'QTCOMPMESEX', 'QTCOMPMESAB', 'QTCOMPMEFER'];
            
            $this->connection->beginTransaction();
            
            foreach ($periodoCard as $diaCard) { // para cada dia do cardapio
                $diaAtual = substr($diaCard['DTCARDIAPME'], 0, 10);
                $diaSemana = $diaCard['WEEKDAY'];
                $idcardespec = ($possuiAgregadaEsp == true) ? 'S' : 'N';
                $feriado = $this->verificaFeriados($cdfilial, $diaAtual); // verifico se é feriado
                $naoPossuiFeriado = (count($feriado) == 0) ? true : false;
                $diaDoComensal = $arrayComensais[strval($diaSemana)];
                $valorDoDia = (isset($comensaisDoDia[0][$diaDoComensal])) ? $comensaisDoDia[0][$diaDoComensal] : 0;
                
                if ($valorDoDia == 0) // se o comensal do dia for 0, não irá inserir
                    continue;
                  
                if ($nrseqcnfcpm == NULL || $nrordcnfcpme == NULL) {
                      $prato = $row['PRATOSELECIONADO'];
                      $configPrato = $this->buscaConfigCard($prato);
                      $nrseqcnfcpm = $configPrato[0]['NRSEQCNFCPME'];
                      $nrordcnfcpme = $configPrato[0]['NRORDCNFCPME'];
                }
                
               if ($naoPossuiFeriado) { // o dia não é feriado, já que não retornou registro
                   $cardapio = $this->utilitiesCard->getCadiaPme($cdfilcentpme, $nrcardpme, $cdservfili, $diaAtual, $cdfilial);
                   $possuiCardapio = (count($cardapio) == 0) ? false : true;
                   $vrpeconscpm = $this->verificaExcecaoComensais($excecoes, $diaCard, $comensaisDiaOuPrato, $comensaisNovosDoPrato, $valorDoDia);
                   
                   if ($possuiCardapio) { // se possui cardapio no dia atual
                       $nrcardiapme = $cardapio[0]['NRCARDIAPME'];
                        
                       if(!$sobreporValue){ // se opcao sobrepor estivar desativada
                         $confCard = $this->possuiComposCard($cdfilcentpme, $nrcardpme, $nrcardiapme, $nrseqcnfcpm); // verifico se possui configuração
                         $naoPossuiConfCard = (count($confCard) == 0) ? true : false;
                         
                         if ($naoPossuiConfCard)
                            $this->utilitiesCard->insertCompCard($cdfilcentpme, $nrcardpme, $nrcardiapme, $nrseqcnfcpm, $cdfilauxprat, $cdprat, $cdtipoprat, $vrpeconscpm, $nrordcnfcpme);
                         else 
                            continue;
                       } else {// se opcao sobrepor estivar marcada
                         if ($possuiAgregadaEsp)
                            $this->utilitiesCard->updateCardiaPme($idcardespec, $cdfilcentpme, $nrcardpme, $nrcardiapme, $cdservfili, $diaAtual, $filial);
                         $this->utilitiesCard->deleteCompCard($cdfilcentpme, $nrcardpme, $nrcardiapme, $nrseqcnfcpm, $cdtipoprat);
                         $this->utilitiesCard->insertCompCard($cdfilcentpme, $nrcardpme, $nrcardiapme, $nrseqcnfcpm, $cdfilauxprat, $cdprat, $cdtipoprat, $vrpeconscpm, $nrordcnfcpme);
                       }
                       
                   } else { // se nao possui cardapio no dia atual
                      $novoPrato = $row['PRATOSELECIONADO'];
                      $novoPrato['DTCARDIAPME'] = $diaAtual;
                      $novoPrato['WEEKDAY'] = $diaSemana;
                      $novoPrato['IDCARDESPEC'] = $idcardespec;
                      $newCardiaPme = $this->elaboracaoCardapio->insertInCardia($novoPrato);
                      $nrcardpme = $newCardiaPme[0]['NRCARDPME'];
                      $nrcardiapme = $newCardiaPme[0]['NRCARDIAPME'];
                      
    			      $this->utilitiesCard->insertCompCard($cdfilcentpme, $nrcardpme, $nrcardiapme, $nrseqcnfcpm, $cdfilauxprat, $cdprat, $cdtipoprat, $vrpeconscpm, $nrordcnfcpme);
                   }
                } else { 
                    continue; // o dia é feriado, então não deve ser feito nada
                }
            }
            
            $this->connection->commit();
            $response->addDataSet(new DataSet("success", array(true)));
        } catch (\Exception $e) {
            $this->connection->rollback();
	        $response->setError(new Error($e->getMessage(), 500));
        }
	}
	
	public function realizaAcaoCopiaComensais(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $prato = $row['PRATOSELECIONADO'];
            $cdfilcentpme = $prato['CDFILCENTPME'];
            $cdservfili = $prato['CDSERVFILI'];
            $cdfilial = $prato['CDFILIAL'];
            $nrcardpme = $prato['NRCARDPME'];
            $comensaisOriginaisDoDia = $prato['QTCOMESTCPME'];
            $nrseqpracpme = $prato['NRSEQPRACPME'];
            $comensaisNovosDoPrato = $row['COMENSAISPRATO'];
            $periodoCard = $row['PERIODOCARD'];
            $excecoes = $row['EXCECOES'];
            $cdtipocard = $row['CDTIPOCARD'];
            $possuiAgregadaEsp = ($prato['filterData']['aggregated']) ? true : false;
            $filial = $possuiAgregadaEsp ? $cdfilial : NULL;
            
            $arrayComensais = ['QTCOMPMEDOM', 'QTCOMPMESEG', 'QTCOMPMETER', 'QTCOMPMEQUA', 'QTCOMPMEQUI', 'QTCOMPMESEX', 'QTCOMPMESAB', 'QTCOMPMEFER'];
            $comensaisDoDia = $this->buscaComensaisDiariosDaSemana($cdfilcentpme, $cdtipocard, $cdservfili, $filial);
            
            $this->connection->beginTransaction();
            
            foreach ($periodoCard as $diaCard) {    // para cada dia do cardapio
                $diaAtual = substr($diaCard['DTCARDIAPME'], 0, 10);
                $diaSemana = $diaCard['WEEKDAY'];
                $diaDoComensal = $arrayComensais[strval($diaSemana)];
                $valorDoDia = (isset($comensaisDoDia[0][$diaDoComensal])) ? $comensaisDoDia[0][$diaDoComensal] : 0;
                
                if ($valorDoDia == 0)
                    continue;
                
                $cardapio = $this->utilitiesCard->getCadiaPme($cdfilcentpme, $nrcardpme, $cdservfili, $diaAtual, $cdfilial);
                $possuiCardapio = (count($cardapio) == 0) ? false : true;
                
                if ($possuiCardapio) {  // verifico se possui cardapio
                
                    if ($nrseqpracpme == NULL) {
                      $configPrato = $this->buscaConfigCard($prato);
                      $nrseqpracpme = $configPrato[0]['NRSEQCNFCPME'];
                    }
                    
                    $vrpeconscpm = $this->verificaExcecaoComensais($excecoes, $diaCard, 'prato', $comensaisNovosDoPrato, $valorDoDia);
                    $nrcardiapme = $cardapio[0]['NRCARDIAPME'];
                    
                    $this->atualizaPrato($vrpeconscpm, $cdfilcentpme, $nrcardpme, $nrcardiapme, $nrseqpracpme);  // atualizo o percentual de comensais
                } else {
                    continue;
                }
            }
            
            $this->connection->commit();
            $response->addDataSet(new DataSet("success", array(true)));
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
	}
	
	// retorna a porcentagem de acordo com a excecao do dia, do comensal do dia ou do comensal do prato.
	private function verificaExcecaoComensais($excecoes, $diaCard, $comensaisDiaOuPrato, $comensaisNovosDoPrato, $valorDoDia) {
	    $diaSemana = strval($diaCard['WEEKDAY']);
	    $possuiComensaisDia = ($comensaisDiaOuPrato == 'dia') ? true : false;
	    
        if ($possuiComensaisDia) {
            if (\Zeedhi\Framework\Util\Functions::arrayKeyExists($diaSemana, $excecoes)) {
                $excecaoDoDia = $excecoes[$diaSemana];
                if ($excecaoDoDia == 0) return 0;
                return (($excecaoDoDia/$valorDoDia)*100);
            } else 
                return 100;
	    } else {
	        if (\Zeedhi\Framework\Util\Functions::arrayKeyExists($diaSemana, $excecoes)) {
                $excecaoDoDia = $excecoes[$diaSemana];
                if ($excecaoDoDia == 0) return 0;
                return (($excecaoDoDia*100)/$valorDoDia);
            } else if ($comensaisNovosDoPrato == 0)
                return 0;
            else 
                return (($comensaisNovosDoPrato*100)/$valorDoDia);
	    }
	}
	
	private function atualizaPrato($vrpeconscpm, $cdfilcentpme, $nrcardpme, $nrcardiapme, $nrseqpracpme) {
		
        $query = "UPDATE COMPCARDPME
                          SET VRPECONSCPME = :VRPECONSCPME
                         WHERE CDFILCENTPME = :OLD_CDFILCENTPME
                          AND NRCARDPME = :OLD_NRCARDPME
                          AND NRCARDIAPME = :OLD_NRCARDIAPME
                          AND NRSEQPRACPME = :OLD_NRSEQPRACPME";

	    
        $params = array(
            "OLD_CDFILCENTPME" => $cdfilcentpme,
            "OLD_NRCARDPME"    => $nrcardpme,
            "OLD_NRCARDIAPME"  => $nrcardiapme,
            "OLD_NRSEQPRACPME" => $nrseqpracpme,
            "VRPECONSCPME"     => $vrpeconscpm
        );
        
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
    }
	
	private function verificaFeriados($filial, $date) {
        $query = "SELECT DTFERIFILI, DSFERIFILI
                FROM FERIFILI
                WHERE (CDFILIAL   = :P_CDFILIAL) AND
                      (DTFERIFILI = :P_DTCARDIAPME)";
                      
    	$params = array(
            "P_CDFILIAL" => $filial,
            "P_DTCARDIAPME"  => $date,
        );
        
        return $this->connection->fetchAll($query, $params);
	}
	
	private function buscaConfigCard($prato) {
		$sql = "SELECT C.NRSEQCNFCPME, C.NRORDCNFCPME FROM CONFCARDPME C WHERE C.CDFILCENTPME =:P_CDFILCENTPME AND C.CDSERVFILI =:P_CDSERVFILI AND C.NRCARDPME =:P_NRCARDPME AND C.CDTIPOPRAT = :P_CDTIPOPRAT";
		$paramSelect = array(
            "P_CDFILCENTPME" => $prato['CDFILCENTPME'],
            "P_NRCARDPME"    => $prato['NRCARDPME'],
            "P_CDSERVFILI"   => $prato['CDSERVFILI'],
            "P_CDTIPOPRAT"   => $prato['CDTIPOPRAT']
        );
      return $this->connection->fetchAll($sql, $paramSelect);
	}
	
	private function possuiComposCard($cdfilcentpme, $nrcardpme, $nrcardiapme, $nrseqcnfcpm) {
    	$sql = "SELECT CDFILCENTPME FROM COMPCARDPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND NRCARDIAPME = :P_NRCARDIAPME AND NRSEQPRACPME = :P_NRSEQPRACPME";
    	$paramSelect = array(
            "P_CDFILCENTPME" => $cdfilcentpme,
            "P_NRCARDPME"    => $nrcardpme,
            "P_NRCARDIAPME"  => $nrcardiapme,
            "P_NRSEQPRACPME" => $nrseqcnfcpm
    	);
        return $this->connection->fetchAll($sql, $paramSelect);
	}
	
	private function buscaComensaisDiariosDaSemana($cdfilcentpme, $cdtipocard, $cdservfili, $cdfilial) {
    	$query = "SELECT C.CDSERVFILI,
                   SUM(QTCOMPMESEG) AS QTCOMPMESEG, SUM(QTCOMPMETER) AS QTCOMPMETER, SUM(QTCOMPMEQUA) AS QTCOMPMEQUA, SUM(QTCOMPMEQUI) AS QTCOMPMEQUI, SUM(QTCOMPMESEX) AS QTCOMPMESEX, SUM(QTCOMPMESAB) AS QTCOMPMESAB, SUM(QTCOMPMEDOM) AS QTCOMPMEDOM, SUM(QTCOMPMEFER) AS QTCOMPMEFER,
                   SUM(QTCLIPMESEG) AS QTCLIPMESEG, SUM(QTCLIPMETER) AS QTCLIPMETER, SUM(QTCLIPMEQUA) AS QTCLIPMEQUA, SUM(QTCLIPMEQUI) AS QTCLIPMEQUI, SUM(QTCLIPMESEX) AS QTCLIPMESEX, SUM(QTCLIPMESAB) AS QTCLIPMESAB, SUM(QTCLIPMEDOM) AS QTCLIPMEDOM, SUM(QTCLIPMEFER) AS QTCLIPMEFER,
                   SUM(QTFUNPMESEG) AS QTFUNPMESEG, SUM(QTFUNPMETER) AS QTFUNPMETER, SUM(QTFUNPMEQUA) AS QTFUNPMEQUA, SUM(QTFUNPMEQUI) AS QTFUNPMEQUI, SUM(QTFUNPMESEX) AS QTFUNPMESEX, SUM(QTFUNPMESAB) AS QTFUNPMESAB, SUM(QTFUNPMEDOM) AS QTFUNPMEDOM, SUM(QTFUNPMEFER) AS QTFUNPMEFER,
                   SUM(QTDOAPMESEG) AS QTDOAPMESEG, SUM(QTDOAPMETER) AS QTDOAPMETER, SUM(QTDOAPMEQUA) AS QTDOAPMEQUA, SUM(QTDOAPMEQUI) AS QTDOAPMEQUI, SUM(QTDOAPMESEX) AS QTDOAPMESEX, SUM(QTDOAPMESAB) AS QTDOAPMESAB, SUM(QTDOAPMEDOM) AS QTDOAPMEDOM, SUM(QTDOAPMEFER) AS QTDOAPMEFER
            FROM COMESERVTCAR C, FILITIPOCARD F
            WHERE (C.CDFILCENTPME = :P_CDFILCENTPME) AND
                  (C.CDTIPOCARD   = :P_CDTIPOCARD)   AND
                  (C.CDSERVFILI   = :P_CDSERVFILI)   AND
                  (C.CDFILIAL     = NVL(:P_CDFILIAL, C.CDFILIAL)) AND
                  (F.CDFILCENTPME = C.CDFILCENTPME)  AND
                  (F.CDTIPOCARD   = C.CDTIPOCARD)    AND
                  (F.CDFILIAL     = C.CDFILIAL)      AND
                  (F.IDTCARATFILI = 'S')
            GROUP BY C.CDSERVFILI";
    	
    	$params = array(
    		"P_CDFILCENTPME" => $cdfilcentpme,
    		"P_CDTIPOCARD" 	 => $cdtipocard,
    		"P_CDSERVFILI"   => $cdservfili,
    		"P_CDFILIAL"     => $cdfilial
    	);
    	
    	$stmt = $this->connection->prepare($query);
    	$stmt->execute($params);
    	return $stmt->fetchAll();
	}
}