<?php
namespace PME\Controllers\Custo;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class ConsumoPlanejadoRealizadoController extends \Zeedhi\Framework\Controller\Report {
    protected $entityManager;
    protected $birtStrategy;
    protected $environment;
    protected $newCode;
    protected $connection;
    protected $lista;
    protected $utilities;

    public function __construct(ReportService $reportService, $entityManager, $birtStrategy, $environment, $newCode, 
      $connection, $lista, $utilities) {
      $this->entityManager = $entityManager;
      $this->birtStrategy  = $birtStrategy;
      $this->environment   = $environment;
      $this->newCode       = $newCode;
      $this->connection    = $connection;
      $this->lista         = $lista;
      $this->utilities     = $utilities;

      parent::__construct($reportService);
    }

    protected $reportMapping = array (
      'PME01800' => 
      array (
        'strategy' => 'BIRT',
        'parameterMapping' => 
        array (
          "TAMACNIVEL"   => "P_NRTAMACNIVEL",
          "TAMCODIGO"    => "P_NRTAMCODIGO",
          "NIVEL"        => "P_NIVEL",
          "CDFILCENTPME" => "P_CDFILCENTPME",
          "NRCONSPLAN"   => "P_NRCONSPLAN",
          "LISTASERV"    => "P_CDSERVFILI", 
          "PRODINICIAL"  => "P_CDPRODINIC",
          "PRODFINAL"    => "P_CDPRODFINA",
          "DTDE"         => "P_DTINICIAL",
          "DTATE"        => "P_DTFINAL",
          "ORDENA"       => "P_ORDENA",
          "DIFQTD"       => "IDDIFERENCA",
          "CDPICTPROD"   => "P_CDPICTPROD",
          "AGRUPMASK"    => "P_MASCAGRUP",
          "NRORG"        => "P_NRORG",
          "DIFPERCENT"   => "P_VARIACAO"
        )
      )
    );
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
      $rows = $request->getRow();
      $nrorg = $this->environment->getNrOrg();
      
      set_time_limit(300);
      ini_set('memory_limit', '3072M');
      
      $rows['NRORG'] = $nrorg;
      
      $cdprodIni = isset($rows['PRODINICIAL']['CDARVPROD']) ? $rows['PRODINICIAL']['CDARVPROD'] : 0;
      if(!(isset($rows['PRODFINAL']['CDARVPROD']))) $cdprodFin = 'zzzzzzzzzzz';
      else $cdprodFin = str_pad($rows['PRODFINAL']['CDARVPROD'], 10, 'z');

      $rows['PRODINICIAL'] = $cdprodIni;
      $rows['PRODFINAL'] = $cdprodFin;
      
      $rows['NRCONSPLAN'] = $this->newCode->getCodeByTableName('CONSPLAN'.$rows['CDFILCENTPME'], $nrorg);
      
      $newRow = $this->procedure($rows);
      
      /*$unidadeServico = [];
      
      foreach($rows['SERVICO'] as $row){
        $concatUnidServ = $row['FILISERV'];
        array_push($unidadeServico, $concatUnidServ);
      }
      
      if (empty($rows['SERVICO'])) {
        $unidServ = $this->getUnidServ($rows);
        $rows['LISTASERV'] = $this->lista->geraLista($unidServ, $nrorg);
      } else $rows['LISTASERV'] = $this->lista->geraLista(join("_", $unidadeServico), $nrorg);*/
      
      // $rows['SERVICO'] = $newRow['SERVICO'];
      $rows['SERVICO_LISTA'] = '';
      if (count($rows['CDSERVFILI']) == 0) {
        $rows['SERVICO_LISTA'] = $this->getUnidServ($rows);
      } else {
        foreach ($rows['CDSERVFILI'] as $servfili) {
          $rows['SERVICO_LISTA'] .= $servfili['FILISERV'].'_';
        }
        
      } 
      
      $rows['LISTASERV'] = $this->lista->geraLista($rows['SERVICO_LISTA'], $nrorg);
      
      if($rows['ORDENA'] == 0) $rows['TAMCODIGO'] = 10;
      else $rows['TAMCODIGO'] = 0;
      
      $this->birtStrategy->setReportLocale($this->utilities->getLanguage());
      $this->birtStrategy->setReportFormat(strtolower($rows['reportFormat']));
    }
    
    //DesvioConsumoController
    //CustoRealizadoGlobalPeriodoController
    public function procedure($rows){
      $nrorg = $this->environment->getNrOrg();
      $cdoperador = $this->environment->getCdOperador(); 
      
      if(empty($rows['PRODINICIAL'])) $cdprodIni = '0';
      else $cdprodIni = $rows['PRODINICIAL'];
      
      if(empty($rows['PRODFINAL'])) $cdprodFin = 'ZZZZZZZZZ';
      else $cdprodFin = str_pad($rows['PRODFINAL'], 10, 'Z');
      
      if(empty($rows['SERVICO']) and !(empty($rows['GRUPOSERVICO']))) {
        $rows['SERVICO'] = $this->getServicosGrupo($rows);
        $servProce = $rows['SERVICO'];
      } else if (empty($rows['SERVICO']) and empty($rows['GRUPOSERVICO'])) {
        $rows['SERVICO'] = null;
        $servProce = null;
      } else {
        if (is_array($rows['SERVICO'])) {
          $servicos = [];
          $unidServ = [];
          foreach($rows['SERVICO'] as $row){
            $concatServ = $row['CDSERVFILI'];
            $concatUnid = $row['CDFILIAL'];
            array_push($servicos, $concatServ);
            array_push($unidServ, $concatUnid);
          }
          $rows['SERVICO'] = join("_", $servicos);
          $rows['UNIDADE'] = join("_", array_unique($unidServ));
        } 
        
        $servProce = $rows['SERVICO'];
      }
      
      if ($rows['GRUPFILI'] == NULL) {
        if (empty($rows['UNIDADE'])) {
          $rows['UNIDADE'] = $this->getUnidades($rows);
          $unidade = $rows['UNIDADE'];
        } else {
          $rows['UNIDADE'] = is_array($rows['UNIDADE']) ? join("_", $rows['UNIDADE']) : $rows['UNIDADE'];
          $unidade = $rows['UNIDADE'];
        }
      } else {
        if (empty($rows['UNIDADE'])) {
          $rows['UNIDADE'] = $rows['UNIDADE'] = $this->getUnidadesGrupo($rows, $nrorg);  
          $unidade = $rows['UNIDADE'];
        }
        else {
          $rows['UNIDADE'] = is_array($rows['UNIDADE']) ? join("_", $rows['UNIDADE']) : $rows['UNIDADE'];
          $unidade = $rows['UNIDADE'];
        }
      }
      
      $tipoCardapio = null;
      if (isset($rows['TIPOCARD'])) {
        if (is_array($rows['TIPOCARD'])) {
          foreach($rows['TIPOCARD'] as $tpCard) {
            $tipoCardapio .= $tpCard['CDTIPOCARD']."_";
          }
        } else {
          $tipoCardapio = $rows['TIPOCARD'];
        }
      }
      
      $connection = $this->entityManager->getConnection();     
      
      for ($i = 0; $i <= strlen($unidade); $i+=3000) {

        $unid = substr($unidade, $i, 3000);
        
        $procedureCons = new StoredProcedure($connection, 'CONSUMO_PLANEJADO_PME'); 
        
        $procedureCons->addParam(new Param('P_CDFILCENTPME', Param::PARAM_INPUT, $rows['CDFILCENTPME'], Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_CDTIPOCARD',   Param::PARAM_INPUT, $tipoCardapio,         Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $unid,                 Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_CDSERVFILI',   Param::PARAM_INPUT, $servProce,            Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $rows['DTDE'],         Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $rows['DTATE'],        Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_IDARQUIVO',    Param::PARAM_INPUT, 'A',                   Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, 'CFP',                 Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, 'N',                   Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, '0',                   Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, 'N',                   Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $rows['NRCONSPLAN'],   Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, $cdprodIni,            Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, $cdprodFin,            Param::PARAM_TYPE_STR));
        $procedureCons->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $cdoperador,           Param::PARAM_TYPE_STR));
        $procedureCons->execute();
      }
      return $rows;
    }
    
    // DesvioConsumoController
    // CustoPlanejadoRealizadoController
    // ConsPorServico
    public function getUnidades($rows){
      $cdoperador = $this->environment->getCdOperador(); 
      $unidades = [];
      //$query = "SELECT DISTINCT FI.CDFILIAL, FI.NMFILIAL, FI.NRORG FROM FILIAL FI, FILIOPER FO WHERE (FI.IDFILCENTPME = 'N') AND (FI.CDFILCENTPME = :CDFILCENTPME) AND (FI.CDFILIAL = FO.CDFILIAL) AND (INSTR(:CDOPERADOR, FO.CDOPERADOR) <> 0 OR :CDOPERADOR IS NULL) ORDER BY FI.CDFILIAL";
      $query = "SELECT DISTINCT FI.CDFILIAL, FI.NMFILIAL, FI.NRORG FROM FILIAL FI, FILIOPER FO WHERE (FI.IDFILCENTPME = 'N') AND (FI.CDFILCENTPME = :CDFILCENTPME) AND (FI.CDFILIAL = FO.CDFILIAL) AND :CDOPERADOR = FO.CDOPERADOR ORDER BY FI.CDFILIAL";
      $params = array(
        "CDFILCENTPME" => $rows["CDFILCENTPME"],
        "CDOPERADOR"   => $cdoperador
      );
      $stmt = $this->connection->prepare($query);
		  $stmt->execute($params);
      $result = $stmt->fetchAll();

      foreach($result as $unid){
        array_push($unidades, $unid['CDFILIAL']);
      }
      $resultado = join("_", $unidades);
      
      return $resultado;
    }
    
    //DesvioConsumoController
    //CustoPlanejadoRealizadoController
    public function getUnidadesGrupo($rows, $nrorg){
      if (is_array($rows['GRUPFILI'])) $grupoUnid = $rows['GRUPFILI']['CDGRUPFILI'];
      else $grupoUnid = $rows['GRUPFILI'];
      
      $cdoperador = $this->environment->getCdOperador(); 
      $unidGrupo = [];
      $query = "SELECT DISTINCT F.CDFILIAL, F.NMFILIAL, F.CDFILIAL || ' - ' || F.NMFILIAL AS DESCRICAO, F.NRORG, F.CDFILCENTPME FROM FILIAL F LEFT JOIN FILGRUPFILI FL ON ( FL.CDGRUPFILI =:CDGRUPFILI OR :CDGRUPFILI = 'T' ) AND FL.NRORG = F.NRORG AND FL.CDFILIAL = F.CDFILIAL WHERE ( FL.CDGRUPFILI IS NOT NULL OR :CDGRUPFILI = 'T' ) AND F.CDFILCENTPME =:CDFILCENTPME AND F.NRORG =:NRORG ORDER BY CDFILIAL";
      $params = array(
        "NRORG"        => $nrorg,
        "CDFILCENTPME" => $rows["CDFILCENTPME"],
        "CDGRUPFILI"   => $grupoUnid
      );
      $stmt = $this->connection->prepare($query);
		  $stmt->execute($params);
      $result = $stmt->fetchAll();

      foreach($result as $unid){
        array_push($unidGrupo, $unid['CDFILIAL']);
      }
      $resultado = join("_", $unidGrupo);
      return $resultado;
    }
    
    private function getServicosGrupo($rows) {
      $nrorg = $this->environment->getNrOrg();
      $servicos = [];
      
      if(is_array($rows['GRUPOSERVICO'])) $grupServ = join("_", $rows['GRUPOSERVICO']);
      else $grupServ = $rows['GRUPOSERVICO'];

      $query = "SELECT S.CDSERVFILI, S.NMSERVICO, S.NRORG 
                  FROM SERVICO S
                 WHERE ( ( INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0 ) OR ( :CDGRUPSERV = 'T' ) ) 
                   AND ( S.CDFILIAL = :CDFILCENTPME ) AND S.IDTIPOSERV = 'M'
                 ORDER BY S.CDSERVFILI";
      
      $params = array(
        "CDFILCENTPME" => $rows["CDFILCENTPME"],
        "CDGRUPSERV"   => $rows['GRUPOSERVICO']
      );
      $stmt = $this->connection->prepare($query);
  	  $stmt->execute($params);
      $result = $stmt->fetchAll();
      
      $servGrupo = [];
      foreach($result as $serv){
        array_push($servGrupo, $serv['CDSERVFILI']);
      }
      $resultado = join("_", $servGrupo);
      return $resultado;
    }
    
    //CustoRealizadoglobalPeriodoController
    public function getUnidServ($rows){
      $nrorg = $this->environment->getNrOrg();
      $unidServ = [];
      
      if ($rows['GRUPFILI'] == NULL) {
        if (empty($rows['UNIDADE'])) $rows['UNIDADE'] = $this->getUnidades($rows);
        else  $rows['UNIDADE'] = is_array($rows['UNIDADE']) ? join("_", $rows['UNIDADE']) : $rows['UNIDADE'];
      } else if ($rows['GRUPFILI'] !== NULL) {
        if (empty($rows['UNIDADE'])) $rows['UNIDADE'] = $this->getUnidadesGrupo($rows, $nrorg);  
        else if (is_array($rows['UNIDADE'])) $rows['UNIDADE'] = join("_", $rows['UNIDADE']);
      }
      if (empty($rows['GRUPOSERVICO'])) $rows['GRUPOSERVICO'] = 'T';
      if (empty($rows['CDGRUPFILI'])) $rows['CDGRUPFILI'] = 'T';
      
      $query = "SELECT DISTINCT S.CDFILIAL, S.CDSERVFILI, S.NMSERVICO, 
                      ( S.CDFILIAL || S.CDSERVFILI ) AS FILISERV, S.NRORG
                  FROM SERVICO      S, FILGRUPFILI  F, FILIAL       FI 
                WHERE ( S.CDFILIAL = F.CDFILIAL (+) ) 
                  AND ( FI.CDFILIAL = S.CDFILIAL ) 
                  AND ( ( INSTR(:CDFILIAL, S.CDFILIAL) <> 0 ) OR ( :CDFILIAL = 'T' ) ) 
                  AND ( ( INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0 ) OR ( :CDGRUPSERV = 'T' ) ) 
                  AND ( ( INSTR(:CDGRUPFILI, F.CDGRUPFILI) <> 0 ) OR ( :CDGRUPFILI = 'T' ) ) 
                  AND ( FI.CDFILCENTPME = :CDFILCENTPME ) 
               ORDER BY S.CDFILIAL, S.CDSERVFILI";
      
      for ($i = 0; $i <= strlen($rows['UNIDADE']); $i+=3000) {
        $unid = substr($rows['UNIDADE'], $i, 3000);
        $params = array(
          "CDFILCENTPME" => $rows["CDFILCENTPME"],
          "CDGRUPFILI"   => $rows['CDGRUPFILI'],
          "CDFILIAL"     => $unid,
          "CDGRUPSERV"   => $rows['GRUPOSERVICO']
        );
        
        $stmt = $this->connection->prepare($query);
  		  $stmt->execute($params);
        $result = $stmt->fetchAll();
        $arrayResult = [];
        foreach($result as $posicao) {
          $arrayResult[] = $posicao;
        }
      }
      
      foreach($arrayResult as $row) {
        array_push($unidServ, $row['FILISERV']);
      }
      $resultadoUnidServ = join("_", $unidServ);

      return $resultadoUnidServ;
    }
    
}